@@ -39,9 +39,6 @@ extern audio_output audio_ao;
 #ifdef CONFIG_SOUNDIO
 extern audio_output audio_soundio;
 #endif
-#ifdef CONFIG_PULSE
-extern audio_output audio_pulse;
-#endif
 #ifdef CONFIG_PA
 extern audio_output audio_pa;
 #endif
@@ -82,9 +79,6 @@ static audio_output *outputs[] = {
 #endif
 #ifdef CONFIG_DUMMY
     &audio_dummy,
-#endif
-#ifdef CONFIG_PULSE
-    &audio_pulse,
 #endif
     NULL};
 
