@@ -48,67 +48,16 @@ static int init(int argc, char **argv) {
   int driver = ao_default_driver_id();
   ao_option *ao_opts = NULL;
 
+  // set up default values first
+
   config.audio_backend_buffer_desired_length = 1.0;
   config.audio_backend_latency_offset = 0;
 
-  // get settings from settings file first, allow them to be overridden by command line options
+  // get settings from settings file first, allow them to be overridden by
+  // command line options
 
-  if (config.cfg != NULL) {
-    /* Get the desired buffer size setting. */
-    if (config_lookup_int(config.cfg, "ao.audio_backend_buffer_desired_length", &value)) {
-      if ((value < 0) || (value > 66150)) {
-        inform("The setting ao.audio_backend_buffer_desired_length is deprecated. "
-               "Use ao.audio_backend_buffer_desired_length_in_seconds instead.");
-        die("Invalid ao audio backend buffer desired length \"%d\". It "
-            "should be between 0 and "
-            "66150, default is 6615",
-            value);
-      } else {
-        inform("The setting ao.audio_backend_buffer_desired_length is deprecated. "
-               "Use ao.audio_backend_buffer_desired_length_in_seconds instead.");
-        config.audio_backend_buffer_desired_length = 1.0 * value / 44100;
-      }
-    }
-
-    /* Get the desired buffer size setting. */
-    if (config_lookup_float(config.cfg, "ao.audio_backend_buffer_desired_length_in_seconds",
-                            &dvalue)) {
-      if ((dvalue < 0) || (dvalue > 1.5)) {
-        die("Invalid ao audio backend buffer desired time \"%f\". It "
-            "should be between 0 and "
-            "1.5, default is 0.15 seconds",
-            dvalue);
-      } else {
-        config.audio_backend_buffer_desired_length = dvalue;
-      }
-    }
-
-    /* Get the latency offset. */
-    if (config_lookup_int(config.cfg, "ao.audio_backend_latency_offset", &value)) {
-      if ((value < -66150) || (value > 66150)) {
-        inform("The setting ao.audio_backend_latency_offset is deprecated. "
-               "Use ao.audio_backend_latency_offset_in_seconds instead.");
-        die("Invalid ao audio backend buffer latency offset \"%d\". It "
-            "should be between -66150 and +66150, default is 0",
-            value);
-      } else {
-        inform("The setting ao.audio_backend_latency_offset is deprecated. "
-               "Use ao.audio_backend_latency_offset_in_seconds instead.");
-        config.audio_backend_latency_offset = 1.0 * value / 44100;
-      }
-    }
-
-    /* Get the latency offset. */
-    if (config_lookup_float(config.cfg, "ao.audio_backend_latency_offset_in_seconds", &dvalue)) {
-      if ((dvalue < -1.0) || (dvalue > 1.5)) {
-        die("Invalid ao audio backend buffer latency offset time \"%f\". It "
-            "should be between -1.0 and +1.5, default is 0 seconds",
-            dvalue);
-      } else {
-        config.audio_backend_latency_offset = dvalue;
-      }
-    }
-  }
+  // do the "general" audio  options. Note, these options are in the "general" stanza!
+  parse_general_audio_options();
 
   optind = 1; // optind=0 is equivalent to optind=1 plus special behaviour
   argv--;     // so we shift the arguments to satisfy getopt()
@@ -169,9 +118,7 @@ static void deinit(void) {
   ao_shutdown();
 }
 
-static void start(int sample_rate, int sample_format) {
-  // this information is not used
-}
+static void start(int sample_rate, int sample_format) {}
 
 static void play(short buf[], int samples) { ao_play(dev, (char *)buf, samples * 4); }
 
