@@ -2,11 +2,13 @@
 #define _COMMON_H
 
 #include <libconfig.h>
+#include <signal.h>
 #include <stdint.h>
 #include <sys/socket.h>
 
 #include "audio.h"
 #include "config.h"
+#include "definitions.h"
 #include "mdns.h"
 
 #if defined(__APPLE__) && defined(__MACH__)
@@ -32,6 +34,13 @@
 #define SAFAMILY sa_family
 #endif
 
+#if defined(HAVE_DBUS) || defined(HAVE_MPRIS)
+enum dbus_session_type {
+  DBT_system = 0, // use the session bus
+  DBT_session,    // use the system bus
+} dbt_type;
+#endif
+
 enum endian_type {
   SS_LITTLE_ENDIAN = 0,
   SS_PDP_ENDIAN,
@@ -41,7 +50,7 @@ enum endian_type {
 enum stuffing_type {
   ST_basic = 0, // straight deletion or insertion of a frame in a 352-frame packet
   ST_soxr,      // use libsoxr to make a 352 frame packet one frame longer or shorter
-} type;
+} s_type;
 
 enum playback_mode_type {
   ST_stereo = 0,
@@ -72,6 +81,8 @@ enum sps_format_t {
 
 typedef struct {
   config_t *cfg;
+  double airplay_volume; // stored here for reloading when necessary
+  char *appName; // normally the app is called shairport-syn, but it may be symlinked
   char *password;
   char *service_name; // the name for the shairport service, e.g. "Shairport Sync Version %v running
                       // on host %h"
@@ -157,6 +168,12 @@ typedef struct {
   int loudness;
   float loudness_reference_volume_db;
   int alsa_use_playback_switch_for_mute;
+#if defined(HAVE_DBUS)
+  enum dbus_session_type dbus_service_bus_type;
+#endif
+#if defined(HAVE_MPRIS)
+  enum dbus_session_type mpris_service_bus_type;
+#endif
 
 } shairport_cfg;
 
@@ -219,4 +236,6 @@ void command_set_volume(double volume);
 void shairport_shutdown();
 // void shairport_startup_complete(void);
 
+extern sigset_t pselect_sigset;
+
 #endif // _COMMON_H
