@@ -104,6 +104,22 @@ void mdns_unregister(void) {
   }
 }
 
+void mdns_dacp_monitor(rtsp_conn_info *conn) {
+  if ((config.mdns) && (config.mdns->mdns_dacp_monitor)) {
+    int error = config.mdns->mdns_dacp_monitor(conn);
+    if (error) {
+      debug(1, "Error starting a DACP monitor.");
+    }
+  } else
+    debug(1, "Can't start a DACP monitor.");
+}
+
+void mdns_dacp_dont_monitor(rtsp_conn_info *conn) {
+  if ((config.mdns) && (config.mdns->mdns_dacp_dont_monitor)) {
+    config.mdns->mdns_dacp_dont_monitor(conn);
+  } else
+    debug(1, "Can't stop a DACP monitor.");
+}
 void mdns_ls_backends(void) {
   mdns_backend **b = NULL;
   printf("Available mDNS backends: \n");
