@@ -2,17 +2,24 @@
 #define _MDNS_H
 
 #include "config.h"
+#include <stdint.h>
+#include <player.h>
 
 extern int mdns_pid;
 
 void mdns_unregister(void);
 void mdns_register(void);
+void mdns_dacp_monitor(rtsp_conn_info *conn);
+void mdns_dacp_dont_monitor(rtsp_conn_info *conn);
+
 void mdns_ls_backends(void);
 
 typedef struct {
   char *name;
   int (*mdns_register)(char *apname, int port);
   void (*mdns_unregister)(void);
+  int (*mdns_dacp_monitor)(rtsp_conn_info *conn);
+  void (*mdns_dacp_dont_monitor)(rtsp_conn_info *conn);
 } mdns_backend;
 
 #ifdef CONFIG_METADATA
